# Generated from SqlBase.g4 by ANTLR 4.7.1
from antlr4 import *

# This class defines a complete listener for a parse tree produced by SqlBaseParser.
class SqlBaseListener(ParseTreeListener):

    # Enter a parse tree produced by SqlBaseParser#singleStatement.
    def enterSingleStatement(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#singleStatement.
    def exitSingleStatement(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#singleExpression.
    def enterSingleExpression(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#singleExpression.
    def exitSingleExpression(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#statementDefault.
    def enterStatementDefault(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#statementDefault.
    def exitStatementDefault(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#use.
    def enterUse(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#use.
    def exitUse(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#createSchema.
    def enterCreateSchema(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#createSchema.
    def exitCreateSchema(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#dropSchema.
    def enterDropSchema(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#dropSchema.
    def exitDropSchema(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#renameSchema.
    def enterRenameSchema(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#renameSchema.
    def exitRenameSchema(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#createTableAsSelect.
    def enterCreateTableAsSelect(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#createTableAsSelect.
    def exitCreateTableAsSelect(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#createTable.
    def enterCreateTable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#createTable.
    def exitCreateTable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#dropTable.
    def enterDropTable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#dropTable.
    def exitDropTable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#insertInto.
    def enterInsertInto(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#insertInto.
    def exitInsertInto(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#delete.
    def enterDelete(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#delete.
    def exitDelete(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#renameTable.
    def enterRenameTable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#renameTable.
    def exitRenameTable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#renameColumn.
    def enterRenameColumn(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#renameColumn.
    def exitRenameColumn(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#dropColumn.
    def enterDropColumn(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#dropColumn.
    def exitDropColumn(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#addColumn.
    def enterAddColumn(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#addColumn.
    def exitAddColumn(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#createView.
    def enterCreateView(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#createView.
    def exitCreateView(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#dropView.
    def enterDropView(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#dropView.
    def exitDropView(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#call.
    def enterCall(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#call.
    def exitCall(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#grant.
    def enterGrant(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#grant.
    def exitGrant(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#revoke.
    def enterRevoke(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#revoke.
    def exitRevoke(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showGrants.
    def enterShowGrants(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showGrants.
    def exitShowGrants(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#explain.
    def enterExplain(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#explain.
    def exitExplain(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showCreateTable.
    def enterShowCreateTable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showCreateTable.
    def exitShowCreateTable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showCreateView.
    def enterShowCreateView(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showCreateView.
    def exitShowCreateView(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showTables.
    def enterShowTables(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showTables.
    def exitShowTables(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showSchemas.
    def enterShowSchemas(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showSchemas.
    def exitShowSchemas(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showCatalogs.
    def enterShowCatalogs(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showCatalogs.
    def exitShowCatalogs(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showColumns.
    def enterShowColumns(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showColumns.
    def exitShowColumns(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showStats.
    def enterShowStats(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showStats.
    def exitShowStats(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showStatsForQuery.
    def enterShowStatsForQuery(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showStatsForQuery.
    def exitShowStatsForQuery(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showFunctions.
    def enterShowFunctions(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showFunctions.
    def exitShowFunctions(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showSession.
    def enterShowSession(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showSession.
    def exitShowSession(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#setSession.
    def enterSetSession(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#setSession.
    def exitSetSession(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#resetSession.
    def enterResetSession(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#resetSession.
    def exitResetSession(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#startTransaction.
    def enterStartTransaction(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#startTransaction.
    def exitStartTransaction(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#commit.
    def enterCommit(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#commit.
    def exitCommit(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#rollback.
    def enterRollback(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#rollback.
    def exitRollback(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#showPartitions.
    def enterShowPartitions(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#showPartitions.
    def exitShowPartitions(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#prepare.
    def enterPrepare(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#prepare.
    def exitPrepare(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#deallocate.
    def enterDeallocate(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#deallocate.
    def exitDeallocate(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#execute.
    def enterExecute(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#execute.
    def exitExecute(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#describeInput.
    def enterDescribeInput(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#describeInput.
    def exitDescribeInput(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#describeOutput.
    def enterDescribeOutput(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#describeOutput.
    def exitDescribeOutput(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#query.
    def enterQuery(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#query.
    def exitQuery(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#with.
    def enterWith(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#with.
    def exitWith(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#tableElement.
    def enterTableElement(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#tableElement.
    def exitTableElement(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#columnDefinition.
    def enterColumnDefinition(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#columnDefinition.
    def exitColumnDefinition(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#likeClause.
    def enterLikeClause(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#likeClause.
    def exitLikeClause(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#properties.
    def enterProperties(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#properties.
    def exitProperties(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#property.
    def enterProperty(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#property.
    def exitProperty(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#queryNoWith.
    def enterQueryNoWith(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#queryNoWith.
    def exitQueryNoWith(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#queryTermDefault.
    def enterQueryTermDefault(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#queryTermDefault.
    def exitQueryTermDefault(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#setOperation.
    def enterSetOperation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#setOperation.
    def exitSetOperation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#queryPrimaryDefault.
    def enterQueryPrimaryDefault(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#queryPrimaryDefault.
    def exitQueryPrimaryDefault(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#table.
    def enterTable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#table.
    def exitTable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#inlineTable.
    def enterInlineTable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#inlineTable.
    def exitInlineTable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#subquery.
    def enterSubquery(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#subquery.
    def exitSubquery(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#sortItem.
    def enterSortItem(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#sortItem.
    def exitSortItem(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#querySpecification.
    def enterQuerySpecification(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#querySpecification.
    def exitQuerySpecification(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#groupBy.
    def enterGroupBy(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#groupBy.
    def exitGroupBy(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#singleGroupingSet.
    def enterSingleGroupingSet(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#singleGroupingSet.
    def exitSingleGroupingSet(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#rollup.
    def enterRollup(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#rollup.
    def exitRollup(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#cube.
    def enterCube(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#cube.
    def exitCube(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#multipleGroupingSets.
    def enterMultipleGroupingSets(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#multipleGroupingSets.
    def exitMultipleGroupingSets(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#groupingExpressions.
    def enterGroupingExpressions(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#groupingExpressions.
    def exitGroupingExpressions(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#groupingSet.
    def enterGroupingSet(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#groupingSet.
    def exitGroupingSet(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#namedQuery.
    def enterNamedQuery(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#namedQuery.
    def exitNamedQuery(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#setQuantifier.
    def enterSetQuantifier(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#setQuantifier.
    def exitSetQuantifier(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#selectSingle.
    def enterSelectSingle(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#selectSingle.
    def exitSelectSingle(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#selectAll.
    def enterSelectAll(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#selectAll.
    def exitSelectAll(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#relationDefault.
    def enterRelationDefault(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#relationDefault.
    def exitRelationDefault(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#joinRelation.
    def enterJoinRelation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#joinRelation.
    def exitJoinRelation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#joinType.
    def enterJoinType(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#joinType.
    def exitJoinType(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#joinCriteria.
    def enterJoinCriteria(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#joinCriteria.
    def exitJoinCriteria(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#sampledRelation.
    def enterSampledRelation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#sampledRelation.
    def exitSampledRelation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#sampleType.
    def enterSampleType(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#sampleType.
    def exitSampleType(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#aliasedRelation.
    def enterAliasedRelation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#aliasedRelation.
    def exitAliasedRelation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#columnAliases.
    def enterColumnAliases(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#columnAliases.
    def exitColumnAliases(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#tableName.
    def enterTableName(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#tableName.
    def exitTableName(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#subqueryRelation.
    def enterSubqueryRelation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#subqueryRelation.
    def exitSubqueryRelation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#unnest.
    def enterUnnest(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#unnest.
    def exitUnnest(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#lateral.
    def enterLateral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#lateral.
    def exitLateral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#parenthesizedRelation.
    def enterParenthesizedRelation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#parenthesizedRelation.
    def exitParenthesizedRelation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#expression.
    def enterExpression(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#expression.
    def exitExpression(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#logicalNot.
    def enterLogicalNot(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#logicalNot.
    def exitLogicalNot(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#booleanDefault.
    def enterBooleanDefault(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#booleanDefault.
    def exitBooleanDefault(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#logicalBinary.
    def enterLogicalBinary(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#logicalBinary.
    def exitLogicalBinary(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#predicated.
    def enterPredicated(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#predicated.
    def exitPredicated(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#comparison.
    def enterComparison(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#comparison.
    def exitComparison(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#quantifiedComparison.
    def enterQuantifiedComparison(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#quantifiedComparison.
    def exitQuantifiedComparison(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#between.
    def enterBetween(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#between.
    def exitBetween(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#inList.
    def enterInList(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#inList.
    def exitInList(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#inSubquery.
    def enterInSubquery(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#inSubquery.
    def exitInSubquery(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#like.
    def enterLike(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#like.
    def exitLike(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#nullPredicate.
    def enterNullPredicate(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#nullPredicate.
    def exitNullPredicate(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#distinctFrom.
    def enterDistinctFrom(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#distinctFrom.
    def exitDistinctFrom(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#valueExpressionDefault.
    def enterValueExpressionDefault(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#valueExpressionDefault.
    def exitValueExpressionDefault(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#concatenation.
    def enterConcatenation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#concatenation.
    def exitConcatenation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#arithmeticBinary.
    def enterArithmeticBinary(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#arithmeticBinary.
    def exitArithmeticBinary(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#arithmeticUnary.
    def enterArithmeticUnary(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#arithmeticUnary.
    def exitArithmeticUnary(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#atTimeZone.
    def enterAtTimeZone(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#atTimeZone.
    def exitAtTimeZone(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#dereference.
    def enterDereference(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#dereference.
    def exitDereference(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#typeConstructor.
    def enterTypeConstructor(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#typeConstructor.
    def exitTypeConstructor(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#specialDateTimeFunction.
    def enterSpecialDateTimeFunction(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#specialDateTimeFunction.
    def exitSpecialDateTimeFunction(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#substring.
    def enterSubstring(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#substring.
    def exitSubstring(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#cast.
    def enterCast(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#cast.
    def exitCast(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#lambda.
    def enterLambda(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#lambda.
    def exitLambda(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#parenthesizedExpression.
    def enterParenthesizedExpression(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#parenthesizedExpression.
    def exitParenthesizedExpression(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#parameter.
    def enterParameter(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#parameter.
    def exitParameter(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#normalize.
    def enterNormalize(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#normalize.
    def exitNormalize(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#intervalLiteral.
    def enterIntervalLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#intervalLiteral.
    def exitIntervalLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#numericLiteral.
    def enterNumericLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#numericLiteral.
    def exitNumericLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#booleanLiteral.
    def enterBooleanLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#booleanLiteral.
    def exitBooleanLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#simpleCase.
    def enterSimpleCase(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#simpleCase.
    def exitSimpleCase(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#columnReference.
    def enterColumnReference(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#columnReference.
    def exitColumnReference(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#nullLiteral.
    def enterNullLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#nullLiteral.
    def exitNullLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#rowConstructor.
    def enterRowConstructor(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#rowConstructor.
    def exitRowConstructor(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#subscript.
    def enterSubscript(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#subscript.
    def exitSubscript(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#subqueryExpression.
    def enterSubqueryExpression(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#subqueryExpression.
    def exitSubqueryExpression(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#binaryLiteral.
    def enterBinaryLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#binaryLiteral.
    def exitBinaryLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#currentUser.
    def enterCurrentUser(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#currentUser.
    def exitCurrentUser(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#extract.
    def enterExtract(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#extract.
    def exitExtract(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#stringLiteral.
    def enterStringLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#stringLiteral.
    def exitStringLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#arrayConstructor.
    def enterArrayConstructor(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#arrayConstructor.
    def exitArrayConstructor(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#functionCall.
    def enterFunctionCall(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#functionCall.
    def exitFunctionCall(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#exists.
    def enterExists(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#exists.
    def exitExists(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#position.
    def enterPosition(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#position.
    def exitPosition(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#searchedCase.
    def enterSearchedCase(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#searchedCase.
    def exitSearchedCase(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#groupingOperation.
    def enterGroupingOperation(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#groupingOperation.
    def exitGroupingOperation(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#basicStringLiteral.
    def enterBasicStringLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#basicStringLiteral.
    def exitBasicStringLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#unicodeStringLiteral.
    def enterUnicodeStringLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#unicodeStringLiteral.
    def exitUnicodeStringLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#timeZoneInterval.
    def enterTimeZoneInterval(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#timeZoneInterval.
    def exitTimeZoneInterval(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#timeZoneString.
    def enterTimeZoneString(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#timeZoneString.
    def exitTimeZoneString(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#comparisonOperator.
    def enterComparisonOperator(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#comparisonOperator.
    def exitComparisonOperator(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#comparisonQuantifier.
    def enterComparisonQuantifier(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#comparisonQuantifier.
    def exitComparisonQuantifier(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#booleanValue.
    def enterBooleanValue(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#booleanValue.
    def exitBooleanValue(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#interval.
    def enterInterval(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#interval.
    def exitInterval(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#intervalField.
    def enterIntervalField(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#intervalField.
    def exitIntervalField(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#normalForm.
    def enterNormalForm(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#normalForm.
    def exitNormalForm(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#type.
    def enterType(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#type.
    def exitType(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#typeParameter.
    def enterTypeParameter(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#typeParameter.
    def exitTypeParameter(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#baseType.
    def enterBaseType(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#baseType.
    def exitBaseType(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#whenClause.
    def enterWhenClause(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#whenClause.
    def exitWhenClause(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#filter.
    def enterFilter(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#filter.
    def exitFilter(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#over.
    def enterOver(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#over.
    def exitOver(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#windowFrame.
    def enterWindowFrame(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#windowFrame.
    def exitWindowFrame(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#unboundedFrame.
    def enterUnboundedFrame(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#unboundedFrame.
    def exitUnboundedFrame(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#currentRowBound.
    def enterCurrentRowBound(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#currentRowBound.
    def exitCurrentRowBound(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#boundedFrame.
    def enterBoundedFrame(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#boundedFrame.
    def exitBoundedFrame(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#explainFormat.
    def enterExplainFormat(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#explainFormat.
    def exitExplainFormat(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#explainType.
    def enterExplainType(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#explainType.
    def exitExplainType(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#isolationLevel.
    def enterIsolationLevel(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#isolationLevel.
    def exitIsolationLevel(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#transactionAccessMode.
    def enterTransactionAccessMode(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#transactionAccessMode.
    def exitTransactionAccessMode(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#readUncommitted.
    def enterReadUncommitted(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#readUncommitted.
    def exitReadUncommitted(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#readCommitted.
    def enterReadCommitted(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#readCommitted.
    def exitReadCommitted(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#repeatableRead.
    def enterRepeatableRead(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#repeatableRead.
    def exitRepeatableRead(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#serializable.
    def enterSerializable(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#serializable.
    def exitSerializable(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#positionalArgument.
    def enterPositionalArgument(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#positionalArgument.
    def exitPositionalArgument(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#namedArgument.
    def enterNamedArgument(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#namedArgument.
    def exitNamedArgument(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#privilege.
    def enterPrivilege(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#privilege.
    def exitPrivilege(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#qualifiedName.
    def enterQualifiedName(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#qualifiedName.
    def exitQualifiedName(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#unquotedIdentifier.
    def enterUnquotedIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#unquotedIdentifier.
    def exitUnquotedIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#quotedIdentifier.
    def enterQuotedIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#quotedIdentifier.
    def exitQuotedIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#backQuotedIdentifier.
    def enterBackQuotedIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#backQuotedIdentifier.
    def exitBackQuotedIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#digitIdentifier.
    def enterDigitIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#digitIdentifier.
    def exitDigitIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#decimalLiteral.
    def enterDecimalLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#decimalLiteral.
    def exitDecimalLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#doubleLiteral.
    def enterDoubleLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#doubleLiteral.
    def exitDoubleLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#integerLiteral.
    def enterIntegerLiteral(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#integerLiteral.
    def exitIntegerLiteral(self, ctx):
        pass


    # Enter a parse tree produced by SqlBaseParser#nonReserved.
    def enterNonReserved(self, ctx):
        pass

    # Exit a parse tree produced by SqlBaseParser#nonReserved.
    def exitNonReserved(self, ctx):
        pass


